import numpy as np
from numba import njit
import time

from scipy.ndimage import uniform_filter1d
from tqdm.autonotebook import tqdm
from piscat.InputOutput.cpu_configurations import CPUConfigurations
from joblib import Parallel, delayed

from PySide2 import QtGui
from PySide2 import QtWidgets
from PySide2 import QtCore

from PySide2.QtCore import *


@njit(parallel=True)
def numba_divide(ndarray_1, ndarray_2):
    divide_result = np.divide(ndarray_1, ndarray_2)
    return divide_result

@njit(parallel=True)
def numba_sum(video, axis):
    sum_result = np.sum(video, axis=axis)
    return sum_result

@njit(parallel=True)
def numba_diff(ndarray_1, ndarray_2):
    diff_result = ndarray_1 -ndarray_2
    return diff_result


class WorkerSignals(QObject):

    updateProgress_DRA = Signal(int)
    result_final = Signal(object)
    finished_DRA = Signal()
    DRA_complete_signal = Signal(bool)


class DifferentialRollingAverage(QRunnable):

    def __init__(self, video=None, batchSize=500, flag_GUI=False, object_update_progressBar=None):
        """
        Differential Rolling Average (DRA).

        Parameters
        ----------
        video: NDArray
            The video is 3D-numpy (number of frames, width, height).

        batchSize: int
            The number of frames in each batch.

        optional_1: GUI
            These flags are used when GUI calls this method.

            * `flag_GUI`: bool
                This flag is defined as True when GUI calls this method.

            * `object_update_progressBar`: object
                Object that updates the progress bar in GUI.
        """
        super(DifferentialRollingAverage, self).__init__()

        self.cpu = CPUConfigurations()

        self.flag_GUI = flag_GUI
        self.DRA_flag_run = True

        self.size_A_diff = (video.shape[0] - 2 * batchSize, video.shape[1], video.shape[2])

        self.video = video.astype(np.float64)
        self.batchSize = batchSize

        self.moving_avg = np.empty_like(self.video)
        self.output_diff = np.empty(self.size_A_diff)

        self.output_batch_1 = np.empty(self.size_A_diff)
        self.fft_output_len = self.video.shape[0] - self.batchSize

        self.object_update_progressBar = object_update_progressBar
        self.threadpool = QThreadPool()
        self.signals = WorkerSignals()

    @Slot()
    def check_DRA_finish(self, flag):
        self.DRA_flag_run = flag

    @Slot()
    def run(self, *args, **kwargs):
        self.signals.DRA_complete_signal.connect(self.check_DRA_finish)
        video_DRA = self.differential_rolling(FFT_flag=False)

        self.signals.result_final.emit(video_DRA)

    def differential_rolling(self, FFT_flag=False):
        """
        To use DRA, you'll need to call the "differential rolling" process.

        Parameters
        ----------
        FFT_flag: bool
            In case it is True, DRA will be performed in parallel to improve the time performance.

        Returns
        -------
        output: NDArray
            Returns DRA video.

        """
        print("\n--- start DRA ---")

        self._apply_moving_average(FFT_flag)

        video = self.output_diff

        output = numba_divide(video, self.output_batch_1)

        return output

    def _apply_moving_average(self, FFT_flag):
        if FFT_flag:
            self.movingAvg_FFT_based()
        else:
            if self.flag_GUI:
                self.DRA_flag_run = True
                self.object_update_progressBar.setLabel('DRA')
            self.temporal_moving_average()

    def temporal_moving_average(self):

        batch_1 = np.sum(self.video[0:self.batchSize, :, :], axis=0)
        batch_2 = np.sum(self.video[self.batchSize:2 * self.batchSize, :, :], axis=0)

        batch_1_ = np.divide(batch_1, self.batchSize)
        batch_2_ = np.divide(batch_2, self.batchSize)

        self.output_diff[0, :, :] = batch_2_ - batch_1_
        self.output_batch_1[0, :, :] = batch_1_

        for i_ in tqdm(range(1, self.video.shape[0] - 2 * self.batchSize)):
            if self.flag_GUI is True:
                self.signals.updateProgress_DRA.emit(i_)
            batch_1 = batch_1 - self.video[i_ - 1, :, :] + self.video[self.batchSize + i_ - 1, :, :]
            batch_2 = batch_2 - self.video[self.batchSize + i_ - 1, :, :] + self.video[(2 * self.batchSize) + i_ - 1, :, :]
            batch_1_ = np.divide(batch_1, self.batchSize)
            batch_2_ = np.divide(batch_2, self.batchSize)

            self.output_diff[i_, :, :] = (batch_2_ - batch_1_)
            self.output_batch_1[i_, :, :] = batch_1_

        if self.flag_GUI is True:
            self.signals.finished_DRA.emit()

    def numba_temporal_moving_average(self):

        batch_1 = numba_sum(self.video[0:self.batchSize, :, :], axis=0)
        batch_2 = numba_sum(self.video[self.batchSize:2 * self.batchSize, :, :], axis=0)

        batch_1_ = numba_divide(batch_1, self.batchSize)
        batch_2_ = numba_divide(batch_2, self.batchSize)

        self.output_diff[0, :, :] = numba_diff(batch_2_, batch_1_)
        self.output_batch_1[0, :, :] = batch_1_

        for i_ in tqdm(range(1, self.video.shape[0] - 2 * self.batchSize)):
            if self.flag_GUI is True:
                self.signals.updateProgress_DRA.emit(i_)
            batch_1 = batch_1 - self.video[i_ - 1, :, :] + self.video[self.batchSize + i_ - 1, :, :]
            batch_2 = batch_2 - self.video[self.batchSize + i_ - 1, :, :] + self.video[(2 * self.batchSize) + i_ - 1, :, :]

            batch_1_ = numba_divide(batch_1, self.batchSize)
            batch_2_ = numba_divide(batch_2, self.batchSize)

            self.output_diff[0, :, :] = numba_diff(batch_2_, batch_1_)
            self.output_batch_1[0, :, :] = batch_1_

        if self.flag_GUI is True:
            self.signals.finished_DRA.emit()

    def movingAvg_FFT_based(self):

        Parallel(n_jobs=self.cpu.n_jobs, backend='threading')(delayed(self.uniform_filter1d_kernel)(r_, c_)
                            for r_ in range(self.video.shape[1])
                            for c_ in range(self.video.shape[2]))

        self.output_diff = self.moving_avg[self.batchSize:self.fft_output_len, ...] - self.moving_avg[0:self.fft_output_len-self.batchSize, ...]

        self.output_batch_1 = self.moving_avg[0:self.fft_output_len - self.batchSize, ...]

    def uniform_filter1d_kernel(self, r_, c_):
        self.moving_avg[:, r_, c_] = uniform_filter1d(self.video[:, r_, c_], size=self.batchSize,
                                                      mode='constant', cval=0.0,
                                                      origin=-(self.batchSize // 2), axis=0)













import math

import numpy as np
from joblib import Parallel, delayed
from matplotlib import pyplot as plt

from piscat.BackgroundCorrection import DRA
from piscat.InputOutput.cpu_configurations import CPUConfigurations
from piscat.Visualization.print_colors import PrintColors
from piscat.Preproccessing.filtering import Filters


class NoiseFloor(CPUConfigurations, PrintColors):

    def __init__(self, video, list_range, n_jobs=None, inter_flag_parallel_active=True):
        """
        This class measures the noise floor for various batch sizes.

        Parameters
        ----------
        video: NDArray
            The video is 3D-numpy (number of frames, width, height).

        list_range: list
            A list of all batch sizes for which DRA should be measured.
        """

        CPUConfigurations.__init__(self)
        PrintColors.__init__(self)

        self.video = video
        self.inter_flag_parallel_active = inter_flag_parallel_active
        self.thr_shot_noise = None
        self.list_range = list_range
        if self.parallel_active and self.inter_flag_parallel_active:
            if n_jobs is not None:
                self.n_jobs = n_jobs
                print("\nThe number of usage CPU cores are {}!".format(n_jobs))

            self.mean = Parallel(n_jobs=self.n_jobs, backend=self.backend, verbose=0)(delayed(self.best_radius_kernel)(i_, flag_parallel=True) for i_ in range(len(self.list_range)))
        else:
            print(f"{self.WARNING}\nThe noise floor is running without parallel loop!{self.ENDC}")

            self.mean = []
            for i_ in range(len(self.list_range)):
                self.mean.append(self.best_radius_kernel(i_, flag_parallel=True))

    def best_radius_kernel(self, i_, flag_parallel):
        DRA_ = DRA.DifferentialRollingAverage(video=self.video, batchSize=self.list_range[i_])
        video_DRA = DRA_.differential_rolling(FFT_flag=False)

        return np.mean(np.std(video_DRA, axis=0))

    def plot_result(self):
        """
        The result of the noise floor is plotted when this function is called.
        """
        fig, ax = plt.subplots()
        ax.plot(self.list_range, self.mean, '--r', label='Experimental result')
        ax.plot(self.list_range, self.mean, 'ro')

        if self.thr_shot_noise is not None:
            ax.plot(self.list_range, self.thr_shot_noise, '--g', label='Theoretical result')
            ax.plot(self.list_range, self.thr_shot_noise, 'go')

        ax.set_xlabel("Batch size")
        ax.set_ylabel("Noise floor")
        ax.legend()
        plt.show()


from PySide2 import QtGui, QtCore, QtWidgets
from PySide2.QtWidgets import *
from PySide2 import QtCore, QtGui
from PySide2.QtGui import *
from PySide2.QtCore import *

import psutil


class CPU_Setting(QtWidgets.QMainWindow):
    update_CPU_Setting = QtCore.Signal()

    def __init__(self):
        super(CPU_Setting,  self).__init__()
        self.n_jobs = None
        self.backend = None
        self.verbose = None
        self.parallel_active = None
        self.threshold_for_parallel_run = None
        self.cpu_configuration_update_flag = True
        self.empty_value_box_flag = False

        self.cpu_backend = {"loky": "loky", "multiprocessing": "multiprocessing", "threading": "threading"}

        self.window = QtWidgets.QWidget()

        self.ok = QtWidgets.QPushButton("Ok")
        self.ok.setAutoDefault(False)
        self.ok.clicked.connect(self.do_update)

        self._ProgressBar = QtWidgets.QProgressBar(self)
        self._ProgressBar.setValue(0)

        self.grid = QtWidgets.QGridLayout()
        self.grid.addWidget(self.createFirstExclusiveGroup(), 0, 0)

        self.grid.addWidget(self.ok, 3, 0)
        # self.grid.addWidget(self._ProgressBar, 3, 1)

        self.setWindowTitle("CPU setting")
        # self.setStyleSheet('QMainWindow{background-color: darkgray;}')
        self.window.setLayout(self.grid)
        self.window.show()

    def createFirstExclusiveGroup(self):
        self.groupBox_CPU = QtWidgets.QGroupBox("CPU parallel setting:")
        self.groupBox_CPU.setCheckable(True)
        self.groupBox_CPU.setChecked(False)
        # groupBox.isChecked(False)
        self.grid2 = QtWidgets.QGridLayout()
        self.add_line_edit1()
        self.groupBox_CPU .setLayout(self.grid2)

        return self.groupBox_CPU

    def setProgress(self, step=1000):
        value = psutil.cpu_percent()

        self._ProgressBar.setMaximum(step * 100)
        self._ProgressBar.setValue(step * value)
        self._ProgressBar.setFormat("%.02f %%" % value)
        self._ProgressBar.setAlignment(Qt.AlignCenter)

    @QtCore.Slot()
    def do_update(self):
        if self.ok.clicked:
            if self.groupBox_CPU.isChecked():
                self.parallel_active = True
                self.get_values()

                if self.n_jobs != '':
                    self.n_jobs = int(self.n_jobs)
                else:
                    self.n_jobs = -1
                    self.msg_box = QtWidgets.QMessageBox()
                    self.msg_box.setWindowTitle("Warning!")
                    self.msg_box.setText("all CPU cores are selected!")
                    self.msg_box.exec_()

                if self.verbose != '':
                    self.verbose = int(self.verbose)
                else:
                    self.verbose = 10
                    self.msg_box = QtWidgets.QMessageBox()
                    self.msg_box.setWindowTitle("Warning!")
                    self.msg_box.setText("Default Verbose is selected!")
                    self.msg_box.exec_()

                if self.backend is None:
                    self.backend = 'multiprocessing'
                    self.msg_box = QtWidgets.QMessageBox()
                    self.msg_box.setWindowTitle("Warning!")
                    self.msg_box.setText("Default backend is selected!")
                    self.msg_box.exec_()

            else:
                self.parallel_active = False

            self.cpu_configuration_update_flag = False
            self.update_CPU_Setting.emit()
            self.window.close()

    def on_select(self):
        if self.combo.currentText() == "-Select the backend-":
            self.msg_box = QtWidgets.QMessageBox()
            self.msg_box.setWindowTitle("Warning!")
            self.msg_box.setText("Please select the backend!")
            self.msg_box.exec_()
        else:
            self.backend = self.cpu_backend[self.combo.currentText()]

    def add_line_edit1(self):
        self.combo = QtWidgets.QComboBox(self)
        self.combo.addItem("-Select the backend-")
        self.combo.addItem("loky")
        self.combo.addItem("multiprocessing")
        self.combo.addItem("threading")

        self.combo.currentIndexChanged.connect(self.on_select)

        self.le1_cpu_cores = QtWidgets.QLineEdit()
        self.le1_cpu_cores.setPlaceholderText('Number of use CPU cores')
        self.le1_cpu_cores_label = QtWidgets.QLabel("Number cores:")

        self.le2_verbose = QtWidgets.QLineEdit()
        self.le2_verbose.setPlaceholderText('Verbose')
        self.le2_verbose_label = QtWidgets.QLabel("Verbose:")

        self.grid2.addWidget(self.combo, 1, 0)
        self.grid2.addWidget(self.le1_cpu_cores_label, 2, 0)
        self.grid2.addWidget(self.le2_verbose_label, 3, 0)

        self.grid2.addWidget(self.combo, 1, 1)
        self.grid2.addWidget(self.le1_cpu_cores, 2, 1)
        self.grid2.addWidget(self.le2_verbose, 3, 1)

    def get_values(self):
        self.on_select()
        try:
            self.n_jobs = self.le1_cpu_cores.text()
            self.verbose = self.le2_verbose.text()
            self.threshold_for_parallel_run = self.le3_threshold_for_parallel_run.text()
            self.empty_value_box_flag = True
        except:
            self.empty_value_box_flag = False

    def closeEvent(self, **kwargs):
        QtCore.QCoreApplication.instance().quit()



from piscat.Analysis.plot_protein_histogram import PlotProteinHistogram

from PySide2 import QtGui, QtCore, QtWidgets


class Histogram_GUI(QtWidgets.QWidget):
    update_tracking = QtCore.Signal(object)
    output_setting_Tab_tracking = QtCore.Signal(object)

    def __init__(self, parent=None):
        super(Histogram_GUI, self).__init__(parent)

        self.input_data = None
        self.lower_limit = None
        self.upper_limit = None
        self.max_n_components = None
        self.minPeak_width = None
        self.n_bins = None

        self.empty_value_box_flag = False

        self.step_limit = 1e-6
        self.setting_tracking = {}

        self.resize(300, 300)
        self.setWindowTitle('Histogram')

        self.plot_hist = QtWidgets.QPushButton("plot_histogram")
        self.plot_hist.clicked.connect(self.do_hist)

        self.le_lower_contrast_trim = QtWidgets.QLineEdit()
        self.le_lower_contrast_trim.setPlaceholderText("lower_limitation")
        self.le_lower_contrast_trim_label = QtWidgets.QLabel("lower limitation (Contrast):")

        self.le_upper_contrast_trim = QtWidgets.QLineEdit()
        self.le_upper_contrast_trim.setPlaceholderText("upper_limitation")
        self.le_upper_contrast_trim_label = QtWidgets.QLabel("upper limitation (Contrast):")

        self.le_MaxGMM_mode = QtWidgets.QLineEdit()
        self.le_MaxGMM_mode.setPlaceholderText("max_n_components for GMM")
        self.le_MaxGMM_mode_label = QtWidgets.QLabel("max n_components:")

        self.le_hist_bin = QtWidgets.QLineEdit()
        self.le_hist_bin.setPlaceholderText("#bins")
        self.le_hist_bin_label = QtWidgets.QLabel("bins:")

        self.min_peak_width = QtWidgets.QLineEdit()
        self.min_peak_width.setPlaceholderText("Min Peak Width")
        self.min_peak_width_label = QtWidgets.QLabel("Min Peak Width:")

        self.grid = QtWidgets.QGridLayout()
        self.grid.addWidget(self.createFirstExclusiveGroup(), 0, 0)

        self.setLayout(self.grid)

    def __del__(self):
        print('Destructor called, Employee deleted.')

    def createFirstExclusiveGroup(self):
        self.checkbox_GMM_FIT = QtWidgets.QCheckBox("Histogram and GMM", self)

        self.groupBox_hist = QtWidgets.QGroupBox("Histogram and GMM")

        self.grid3 = QtWidgets.QGridLayout()
        self.grid3.addWidget(self.min_peak_width_label, 0, 0)
        self.grid3.addWidget(self.min_peak_width, 0, 1)
        self.grid3.addWidget(self.le_lower_contrast_trim_label, 1, 0)
        self.grid3.addWidget(self.le_lower_contrast_trim, 1, 1)
        self.grid3.addWidget(self.le_upper_contrast_trim_label, 2, 0)
        self.grid3.addWidget(self.le_upper_contrast_trim, 2, 1)
        self.grid3.addWidget(self.le_MaxGMM_mode_label, 3, 0)
        self.grid3.addWidget(self.le_MaxGMM_mode, 3, 1)
        self.grid3.addWidget(self.checkbox_GMM_FIT, 3, 2)
        self.grid3.addWidget(self.le_hist_bin_label, 4, 0)
        self.grid3.addWidget(self.le_hist_bin, 4, 1)
        self.grid3.addWidget(self.plot_hist, 5, 0)

        self.groupBox_hist.setLayout(self.grid3)
        return self.groupBox_hist

    def do_hist(self):
        self.step_limit = 1e-6

        self.get_values_histogram_plot()
        if self.empty_value_box_flag:
            if self.checkbox_GMM_FIT.isChecked():
                his_ = PlotProteinHistogram(intersection_display_flag=False)
                his_(folder_name='', particles=self.input_data, batch_size=self.batch_size,
                          video_frame_num=self.number_frames, MinPeakWidth=self.minPeak_width,
                          MinPeakProminence=0)

                his_.plot_histogram(bins=self.n_bins, upper_limitation=self.upper_limit, lower_limitation=self.lower_limit, step_range=self.step_limit,
                                    face='g',
                                    edge='k', Flag_GMM_fit=True, max_n_components=self.max_n_components)
            else:
                his_ = PlotProteinHistogram(intersection_display_flag=False)
                his_(folder_name='', particles=self.input_data, batch_size=self.batch_size,
                     video_frame_num=self.number_frames, MinPeakWidth=self.minPeak_width,
                     MinPeakProminence=0)
                his_.plot_histogram(bins=self.n_bins, upper_limitation=self.upper_limit,
                                    lower_limitation=self.lower_limit, step_range=self.step_limit,
                                    face='g',
                                    edge='k', Flag_GMM_fit=False, max_n_components=self.max_n_components)
            self.empty_value_box_flag = False

    def get_values_histogram_plot(self):

        try:
            self.minPeak_width = float(self.min_peak_width.text())
            self.lower_limit = float(self.le_lower_contrast_trim.text())
            self.upper_limit = float(self.le_upper_contrast_trim.text())
            self.max_n_components = int(self.le_MaxGMM_mode.text())
            self.n_bins = int(self.le_hist_bin.text())

            self.empty_value_box_flag = True

        except:
            self.msg_box3 = QtWidgets.QMessageBox()
            self.msg_box3.setWindowTitle("Warning!")
            self.msg_box3.setText("Please filled all parameters!")
            self.msg_box3.exec_()

            self.empty_value_box_flag = False

    @QtCore.Slot()
    def update_in_data(self, data_in):
        self.input_data = data_in[0]
        self.number_frames = data_in[1]
        self.batch_size = data_in[2]

    def closeEvent(self, event):
        QtCore.QCoreApplication.instance().quit()



import inspect

import piscat.Analysis
import piscat.BackgroundCorrection
import piscat.InputOutput
import piscat.Localization
import piscat.Preproccessing
import piscat.Trajectory
import piscat.Visualization
import piscat.GUI
import multiprocessing
from PyQt5 import QtWidgets
import sys

def test_can_import_piscat_subpackages():

    assert inspect.isclass(piscat.Analysis.PlotProteinHistogram)
    assert inspect.isclass(piscat.Analysis.ReadProteinAnalysis)
    assert inspect.isfunction(piscat.Analysis.protein_analysis)

    assert inspect.isclass(piscat.BackgroundCorrection.DifferentialRollingAverage)
    assert inspect.isclass(piscat.BackgroundCorrection.NoiseFloor)

    assert inspect.isclass(piscat.InputOutput.download_tutorial_data)
    assert inspect.isclass(piscat.InputOutput.CPUConfigurations)
    assert inspect.isclass(piscat.InputOutput.CameraParameters)
    assert inspect.isclass(piscat.InputOutput.DirectoryType)
    assert inspect.isclass(piscat.InputOutput.StatusLine)
    assert inspect.isclass(piscat.InputOutput.Image2Video)

    assert inspect.isclass(piscat.Localization.DirectionalIntensity)
    assert inspect.isclass(piscat.Localization.PSFsExtraction)
    assert inspect.isclass(piscat.Localization.SpatialFilter)
    assert inspect.isclass(piscat.Localization.RadialCenter)
    assert inspect.isclass(piscat.Localization.CPUConfigurations)

    assert inspect.isclass(piscat.Preproccessing.ImagePatching)
    assert inspect.isclass(piscat.Preproccessing.Normalization)
    assert inspect.isclass(piscat.Preproccessing.Filters)
    assert inspect.isclass(piscat.Preproccessing.FFT2D)
    assert inspect.isclass(piscat.Preproccessing.FastRadialSymmetryTransform)
    assert inspect.isclass(piscat.Preproccessing.RadialVarianceTransform)

    assert inspect.isclass(piscat.Trajectory.TemporalFilter)
    assert inspect.isclass(piscat.Trajectory.Linking)
    assert inspect.isfunction(piscat.Trajectory.protein_trajectories_list2dic)

    assert inspect.isclass(piscat.Visualization.Display)
    assert inspect.isclass(piscat.Visualization.JupyterDisplay)
    assert inspect.isclass(piscat.Visualization.JupyterPSFs_localizationPreviewDisplay)
    assert inspect.isclass(piscat.Visualization.DisplayPSFs_subplotLocalizationDisplay)
    assert inspect.isclass(piscat.Visualization.JupyterSelectedPSFs_localizationDisplay)
    assert inspect.isclass(piscat.Visualization.ContrastAdjustment)
    assert inspect.isclass(piscat.Visualization.PrintColors)


def test_gui_can_be_constructed():
    multiprocessing.freeze_support()
    app = QtWidgets.QApplication(sys.argv)
    piscat.GUI.PiSCAT_GUI()
    #sys.exit(app.exec_())  # for checking only



import os
from piscat.Analysis.plot_protein_histogram import PlotProteinHistogram
from piscat.InputOutput import reading_videos
from piscat.InputOutput import read_write_data
from piscat.InputOutput.read_write_data import load_dict_from_hdf5
from piscat.InputOutput.camera_setting import CameraParameters


class ReadProteinAnalysis(CameraParameters):

    def __init__(self, camera_Name='Photonfocus.json'):
        """
        This class is developed to read the results of different video analyses that were analyzed with the ``protein_analysis``
        function. In the end, this class has different methods to display histograms or save histogram results.

        Parameters
        ----------
        camera_Name: str
            Define the name of the camera configuration JSON file that will use for reading pixel size.
        """
        CameraParameters.__init__(self, name=camera_Name)

    def __call__(self, dirName, name_dir, video_frame_num, MinPeakWidth, MinPeakProminence=0, type_file='h5'):
        """
        By calling the object of class this function tries to read the result of the corresponding video it is defined
        with ``dirName`` and ``name_dir``. These results concatenated with previous results to use for plotting histogram.

        Parameters
        ----------
        dirName: str
            Path to the result of video analysis.

        name_dir: str
            The name of a folder that analysis of video was saved on it.

        video_frame_num: int
            The number of frames for the corresponding video.

        MinPeakWidth: int
            This is defined as the minimum V-shaped mouth that will use for prominence.

        MinPeakProminence: int
            This is defined as the minimum V-shape height that will use for prominence.

        type_file: str
            It defines the format of the file as the save file for analysis data ('HDF5', 'Matlab').

        """

        df_histogram = reading_videos.DirectoryType(dirName, type_file=type_file).return_df()
        paths = df_histogram['Directory'].tolist()
        file_names = df_histogram['File'].tolist()

        self.his_ = PlotProteinHistogram(intersection_display_flag=False)

        num_particles = 0
        folder_cnt_ = 0

        for p_, n_ in zip(paths, file_names):

            if name_dir in p_:
                print(p_)
                hyperparameters = None
                PSFs_Particels_num = None
                df_json = reading_videos.DirectoryType(p_, type_file='.json').return_df()
                paths_json = df_json['Directory'].tolist()
                names_json = df_json['File'].tolist()
                for p_json, n_json in zip(paths_json, names_json):
                    if 'flags' in n_json:
                        flags = read_write_data.read_json2dic(path=p_json, name=n_json)

                    elif 'hyperparameters' in n_json:
                        hyperparameters = read_write_data.read_json2dic(path=p_json, name=n_json)

                    elif 'PSFs_Particels_num' in n_json:
                        PSFs_Particels_num = read_write_data.read_json2dic(path=p_json, name=n_json)
                        num_particles = num_particles + PSFs_Particels_num['#Particles_after_V_shapeFilter']

                if hyperparameters is not None:
                    filename = os.path.join(p_, n_)
                    data_dic = load_dict_from_hdf5(filename)
                    if PSFs_Particels_num is not None:
                        if '#Totall_frame_num_DRA' in PSFs_Particels_num.keys():
                            video_frame_num = PSFs_Particels_num['#Totall_frame_num_DRA']
                        else:
                            video_frame_num = video_frame_num

                        self.his_(folder_name=n_, particles=data_dic, batch_size=hyperparameters["batch_size"], video_frame_num=video_frame_num, MinPeakWidth=MinPeakWidth,
                             MinPeakProminence=MinPeakProminence,
                             pixel_size=self.pixelSize)

                        folder_cnt_ += 1

        print('{} folders was read'.format(folder_cnt_))
        print('{} df_PSFs should find in histogram '.format(num_particles))

    def plot_localization_heatmap(self, pixelSize=None, unit='um'):
        """
        This method plots heatmap of particle localization. The size of each disk depicts the movment of each particles during tracking.

        Parameters
        ----------
        pixelSize: float
            camera pixel size
        unit: str
            unit of axises
        """
        if pixelSize is not None:
            self.pixelSize = pixelSize
        self.his_.plot_localization_heatmap(self.pixelSize, unit=unit)


    def plot_hist(self, his_setting):
        """
        This method plots histograms for different contrast extraction methods for black PSFs, white PSFs and all together.

        Parameters
        ----------
        his_setting: dic
             This dictionary defines a histogram plotting setting. In the following you can see the example for it:

                | his_setting = {'bins': None, 'lower_limitation': -7e-4, 'upper_limitation': 7e-4,
                                   'Flag_GMM_fit': True, 'max_n_components': 3, 'step_range': 1e-6,
                                   'face': 'g', 'edge': 'k', 'scale': 1e1, 'external_GMM': False}
        """
        self.his_.plot_histogram(bins=his_setting['bins'], upper_limitation=his_setting['upper_limitation'],
                                lower_limitation=his_setting['lower_limitation'], step_range=his_setting['step_range'],
                                face=his_setting['face'], edge=his_setting['edge'], Flag_GMM_fit=his_setting['Flag_GMM_fit'],
                                max_n_components=his_setting['max_n_components'], scale=his_setting['scale'],
                                external_GMM=his_setting['external_GMM'])

    def plot_hist_2Dfit(self, his_setting):
        """
        This method plots histograms for 2D Gaussian fitting contrast for black PSFs, white PSFs and all together.

        Parameters
        ----------
        his_setting: dic
            This dictionary defines a histogram plotting setting. In the following you can see the example for it:

               | his_setting = {'bins': None, 'lower_limitation': -7e-4, 'upper_limitation': 7e-4,
                                  'Flag_GMM_fit': True, 'max_n_components': 3, 'step_range': 1e-6, 'face': 'g', 'edge': 'k', }
        """
        self.his_.plot_fit_histogram(bins=his_setting['bins'], upper_limitation=his_setting['upper_limitation'],
                            lower_limitation=his_setting['lower_limitation'], step_range=his_setting['step_range'],
                            face=his_setting['face'], edge=his_setting['edge'], Flag_GMM_fit=his_setting['Flag_GMM_fit'],
                            max_n_components=his_setting['max_n_components'])

    def save_hist_data(self, dirName, name_dir, his_setting):
        """
        This function save the histogram data with HDF5 format.

        Parameters
        ----------
        dirName: str
            Path for saving data.

        name_dir: str
            Name that use for saving data.

        his_setting: dic
            This dictionary defines a histogram plotting setting. In the following you can see the example for it:

               | his_setting = {'lower_limitation': -7e-4, 'upper_limitation': 7e-4,
                                  'Flag_GMM_fit': True, 'max_n_components': 3}
        """
        self.his_.save_hist_data(dirName=dirName, name=name_dir, upper_limitation=his_setting['upper_limitation'],
                                 lower_limitation=his_setting['lower_limitation'],
                                 Flag_GMM_fit=his_setting['Flag_GMM_fit'], max_n_components=his_setting['max_n_components'])




import matplotlib.pyplot as plt
from piscat.InputOutput import reading_videos
from PySide2 import QtCore
from PySide2 import QtWidgets


class RawImage(QtWidgets.QMainWindow):

    def __init__(self, fileName):
        super(RawImage, self).__init__()
        self.fileName = fileName

        self.width_size = None
        self.height_size = None

        self.width_size_s = None
        self.width_size_e = None

        self.height_size_s = None
        self.height_size_e = None

        self.frame_s = None
        self.frame_e = None
        self.frame_jump = None

        self.set_bit_order = None
        self.type = None

        self.flag_display = False

        self.little_endian_flag = False
        self.big_endian_flag = False

        self.image_format = {"int_8": "i1", "int_16": "i2", "int_32": "i4", "int_64": "i8", "uint_8": "u1", "uint_16": "u2", "uint_32": "u4",
                             "uint_64": "u8", "float_16": "f2", "float_32": "f4", "float_64": "f8"}
        self.raw_data_update_flag = True

        self.window = QtWidgets.QWidget()

        self.combo = QtWidgets.QComboBox(self)
        self.combo.addItem("-Select the image bin_type-")
        self.combo.addItem("int_8")
        self.combo.addItem("int_16")
        self.combo.addItem("int_32")
        self.combo.addItem("int_64")

        self.combo.addItem("uint_8")
        self.combo.addItem("uint_16")
        self.combo.addItem("uint_32")
        self.combo.addItem("uint_64")

        self.combo.addItem("float_16")
        self.combo.addItem("float_32")
        self.combo.addItem("float_64")
        self.combo.currentIndexChanged.connect(self.on_select)

        self.le1 = QtWidgets.QLineEdit()
        self.le1.setPlaceholderText('width')
        self.le_1_label = QtWidgets.QLabel("Image size width(px):")
        self.le1.setFixedWidth(50)

        self.le2 = QtWidgets.QLineEdit()
        self.le2.setPlaceholderText('height')
        self.le_2_label = QtWidgets.QLabel("Image size height(px):")
        self.le2.setFixedWidth(50)

        self.checkbox_0 = QtWidgets.QRadioButton("little-endian byte order")
        self.checkbox_0.toggled.connect(lambda: self.btnstate(self.checkbox_0))
        self.checkbox_0.setChecked(True)

        self.checkbox_1 = QtWidgets.QRadioButton("big-endian byte order")
        self.checkbox_1.toggled.connect(lambda: self.btnstate(self.checkbox_1))

        self.checkbox_display = QtWidgets.QCheckBox("Display", self)

        self.preview = QtWidgets.QPushButton("Preview")
        self.preview.setAutoDefault(False)
        self.preview.setFixedWidth(100)
        self.preview.clicked.connect(self.do_update_preview)

        self.ok = QtWidgets.QPushButton("Ok")
        self.ok.setAutoDefault(False)
        self.ok.setFixedWidth(100)
        self.ok.clicked.connect(self.do_update)

        self.grid = QtWidgets.QGridLayout()
        self.grid.addWidget(self.createFirstExclusiveGroup(), 0, 0)
        self.grid.addWidget(self.createSecondExclusiveGroup(), 1, 0)
        self.grid.addWidget(self.preview, 5, 0)
        self.grid.addWidget(self.ok, 6, 0)

        self.setWindowTitle("Reading video")
        self.window.setLayout(self.grid)
        self.window.show()

    def createFirstExclusiveGroup(self):
        groupBox = QtWidgets.QGroupBox("Raw information:")
        self.grid1 = QtWidgets.QGridLayout()
        self.grid1.addWidget(self.combo, 0, 0)
        self.grid1.addWidget(self.checkbox_0, 0, 1)
        self.grid1.addWidget(self.checkbox_1, 0, 2)
        self.grid1.addWidget(self.checkbox_display, 0, 3)
        self.grid1.addWidget(self.le_1_label, 1, 0)
        self.grid1.addWidget(self.le1, 1, 1)
        self.grid1.addWidget(self.le_2_label, 2, 0)
        self.grid1.addWidget(self.le2, 2, 1)
        groupBox.setLayout(self.grid1)

        return groupBox

    def createSecondExclusiveGroup(self):
        self.groupBox_cropping = QtWidgets.QGroupBox("Video Cropping")
        self.groupBox_cropping.setCheckable(True)
        self.groupBox_cropping.setChecked(False)
        self.grid2 = QtWidgets.QGridLayout()
        self.info_cut_edith()
        self.groupBox_cropping .setLayout(self.grid2)

        return self.groupBox_cropping

    def do_update_preview(self):
        if self.preview.clicked:
            self.on_select()
            self.get_values()
            self.check_button()
            if self.height_size != '' and self.width_size != '':
                self.width_size = int(self.width_size)
                self.height_size = int(self.height_size)
                self.frame_s = int(0)

            if self.groupBox_cropping.isChecked():

                if self.width_size_s != '' and self.width_size_e != '':
                    self.width_size_s = int(self.width_size_s)
                    self.width_size_e = int(self.width_size_e)
                else:
                    self.msg_box = QtWidgets.QMessageBox()
                    self.msg_box.setWindowTitle("Warning!")
                    self.msg_box.setText("All rows are selected")
                    self.msg_box.exec_()
                    self.width_size_s = 0
                    self.width_size_e = self.width_size

                if self.height_size_s != '' and self.height_size_e != '':
                    self.height_size_s = int(self.height_size_s)
                    self.height_size_e = int(self.height_size_e)
                else:
                    self.msg_box = QtWidgets.QMessageBox()
                    self.msg_box.setWindowTitle("Warning!")
                    self.msg_box.setText("All columns are selected")
                    self.msg_box.exec_()
                    self.height_size_s = 0
                    self.height_size_e = self.height_size

                if self.frame_s != '' and self.frame_e != '' and self.frame_jump != '':
                    self.frame_s = int(self.frame_s)
                    self.frame_e = int(self.frame_e)
                    self.frame_jump = int(self.frame_jump)

                else:
                    self.msg_box = QtWidgets.QMessageBox()
                    self.msg_box.setWindowTitle("Warning!")
                    self.msg_box.setText("All frames are selected")
                    self.msg_box.exec_()
                    self.frame_s = 0
                    self.frame_e = -1
                    self.frame_jump = 1

                # self.check_cut_size_odd()
            try:
                video = reading_videos.read_binary(self.fileName, img_height=self.height_size, img_width=self.width_size,
                                                    image_type=self.set_bit_order + self.type,
                                                    s_frame=self.frame_s, e_frame=self.frame_s+2)
                if self.groupBox_cropping.isChecked():

                    video = video[0:-1, self.width_size_s:self.width_size_e,
                                    self.height_size_s:self.height_size_e]
                plt.figure()
                plt.imshow(video[0, :, :])
                plt.show()
            except:
                self.msg_box = QtWidgets.QMessageBox()
                self.msg_box.setWindowTitle("Warning!")
                self.msg_box.setText("Selected parameters are not correct!")
                self.msg_box.exec_()

    def do_update(self):
        if self.ok.clicked:
            self.on_select()
            self.get_values()
            self.check_button()
            if self.checkbox_display.isChecked():
                self.flag_display = True

            if self.height_size != '' and self.width_size != '':
                self.width_size = int(self.width_size)
                self.height_size = int(self.height_size)

                if self.groupBox_cropping .isChecked():
                    if self.width_size_s != '' and self.width_size_e != '':
                        self.width_size_s = int(self.width_size_s)
                        self.width_size_e = int(self.width_size_e)
                    else:
                        self.msg_box = QtWidgets.QMessageBox()
                        self.msg_box.setWindowTitle("Warning!")
                        self.msg_box.setText("All rows are selected")
                        self.msg_box.exec_()
                        self.width_size_s = 0
                        self.width_size_e = self.width_size

                    if self.height_size_s != '' and self.height_size_e != '':
                        self.height_size_s = int(self.height_size_s)
                        self.height_size_e = int(self.height_size_e)
                    else:
                        self.msg_box = QtWidgets.QMessageBox()
                        self.msg_box.setWindowTitle("Warning!")
                        self.msg_box.setText("All columns are selected")
                        self.msg_box.exec_()
                        self.height_size_s = 0
                        self.height_size_e = self.height_size

                    if self.frame_s != '' and self.frame_e != '' and self.frame_jump != '':
                        self.frame_s = int(self.frame_s)
                        self.frame_e = int(self.frame_e)
                        self.frame_jump = int(self.frame_jump)

                    else:
                        self.msg_box = QtWidgets.QMessageBox()
                        self.msg_box.setWindowTitle("Warning!")
                        self.msg_box.setText("All frames are selected")
                        self.msg_box.exec_()
                        self.frame_s = 0
                        self.frame_e = -1
                        self.frame_jump = 1
                else:
                    self.frame_s = int(0)
                    self.frame_e = int(-1)
                    self.frame_jump = 1


                self.raw_data_update_flag = False
                self.window.close()

            else:
                self.msg_box = QtWidgets.QMessageBox()
                self.msg_box.setWindowTitle("Warning!")
                self.msg_box.setText("Please insert image size!")
                self.msg_box.exec_()

    def check_cut_size_odd(self):

        if (self.width_size_s % 2) != 0:
            self.width_size_s = self.width_size_s + 1

        if (self.width_size_e % 2) != 0:
            self.width_size_e = self.width_size_e - 1

        if (self.height_size_s % 2) != 0:
            self.height_size_s = self.height_size_s + 1

        if (self.height_size_e % 2) != 0:
            self.height_size_e = self.height_size_e - 1

    def on_select(self):
        if self.combo.currentText() == "-Select the image bin_type-":
            self.msg_box = QtWidgets.QMessageBox()
            self.msg_box.setWindowTitle("Warning!")
            self.msg_box.setText("Please select bin_type of image!")
            self.msg_box.exec_()
        else:
            self.type = self.image_format[self.combo.currentText()]

    def check_button(self):
        bit_order = {'native': '=', 'little_endian': '<', 'big_endian': '>'}

        if self.little_endian_flag:
            self.set_bit_order = bit_order["little_endian"]
        elif self.big_endian_flag:
            self.set_bit_order = bit_order["big_endian"]
        else:
            self.set_bit_order = bit_order["native"]

    def info_cut_edith(self):
        self.le3 = QtWidgets.QLineEdit()
        self.le3.setPlaceholderText('width_start')
        self.le_3_label = QtWidgets.QLabel("start width pixel:")

        self.le4 = QtWidgets.QLineEdit()
        self.le4.setPlaceholderText('width_end')
        self.le_4_label = QtWidgets.QLabel("end width pixel:")

        self.le5 = QtWidgets.QLineEdit()
        self.le5.setPlaceholderText('height_start')
        self.le_5_label = QtWidgets.QLabel("start height pixel:")

        self.le6 = QtWidgets.QLineEdit()
        self.le6.setPlaceholderText('height_end')
        self.le_6_label = QtWidgets.QLabel("end height pixel:")

        self.le7 = QtWidgets.QLineEdit()
        self.le7.setPlaceholderText('frame_start')
        self.le_7_label = QtWidgets.QLabel("start frame:")

        self.le8 = QtWidgets.QLineEdit()
        self.le8.setPlaceholderText('frame_end')
        self.le_8_label = QtWidgets.QLabel("end frame:")

        self.le9 = QtWidgets.QLineEdit()
        self.le9.setPlaceholderText('frame_jump')
        self.le_9_label = QtWidgets.QLabel("frame stride:")

        self.grid2.addWidget(self.le_3_label, 1, 0)
        self.grid2.addWidget(self.le_4_label, 1, 2)
        self.grid2.addWidget(self.le_5_label, 2, 0)
        self.grid2.addWidget(self.le_6_label, 2, 2)
        self.grid2.addWidget(self.le_7_label, 3, 0)
        self.grid2.addWidget(self.le_8_label, 3, 2)
        self.grid2.addWidget(self.le_9_label, 3, 4)

        self.grid2.addWidget(self.le3, 1, 1)
        self.grid2.addWidget(self.le4, 1, 3)
        self.grid2.addWidget(self.le5, 2, 1)
        self.grid2.addWidget(self.le6, 2, 3)
        self.grid2.addWidget(self.le7, 3, 1)
        self.grid2.addWidget(self.le8, 3, 3)
        self.grid2.addWidget(self.le9, 3, 5)

    def get_values(self):
        self.width_size = self.le1.text()
        self.height_size = self.le2.text()

        if self.groupBox_cropping.isChecked():

            self.width_size_s = self.le3.text()
            self.width_size_e = self.le4.text()

            self.height_size_s = self.le5.text()
            self.height_size_e = self.le6.text()

            self.frame_s = self.le7.text()
            self.frame_e = self.le8.text()
            self.frame_jump = self.le9.text()

    def btnstate(self, b):

        if b.text() == "little-endian byte order":
            if b.isChecked() == True:
                self.little_endian_flag = True
            else:
                self.little_endian_flag = False

        if b.text() == "big-endian byte order":
            if b.isChecked() == True:
                self.big_endian_flag = True
            else:
                self.big_endian_flag = False

    def closeEvent(self, event):
        QtCore.QCoreApplication.instance().quit()
        print("closing PlaySetting")



from piscat.GUI.Visualization.fun_display_localization import Visulization_localization
from piscat.GUI.InputOutput import Reading

from PySide2 import QtCore, QtWidgets
from functools import partial
import numpy as np


class AnalysisConstant(QtWidgets.QMainWindow):
    update_output = QtCore.Signal(object)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.resize(600, 1000)
        self.flag_analysis_finish = True
        self.flag_remove_box = True

        self.original_video_1 = None
        self.out_video = None
        self.constant_number = None
        self.different_views = {}

        self.combo = QtWidgets.QComboBox(self)
        self.combo.addItem("-Select the analysis-")
        self.combo.addItem("Subtraction")
        self.combo.addItem("ADD")
        self.combo.addItem("Divide")
        self.combo.addItem("Multiplication")
        self.combo.currentIndexChanged.connect(self.on_select)

        self.load1 = QtWidgets.QPushButton("load video")
        self.load1.setAutoDefault(False)
        self.load1.clicked.connect(self.read_data1)
        self.load1.setEnabled(False)
        self.load1.setFixedWidth(100)

        self.inputFileLineEdit1 = QtWidgets.QTextEdit(self)
        self.inputFileLineEdit1.setFixedHeight(20)
        self.inputFileLineEdit1.setFixedWidth(500)

        self.inputNumber = QtWidgets.QLineEdit(self)
        self.inputNumber.setFixedHeight(20)
        self.inputNumber.setFixedWidth(100)
        self.inputNumber_label = QtWidgets.QLabel("Number:")

        self.ok = QtWidgets.QPushButton("Ok")
        self.ok.setAutoDefault(False)
        self.ok.clicked.connect(self.do_update)

        self.grid = QtWidgets.QGridLayout()
        self.grid.addWidget(self.combo, 0, 0)
        self.grid.addWidget(self.load1, 1, 0)
        self.grid.addWidget(self.inputFileLineEdit1, 1, 1)
        self.grid.addWidget(self.inputNumber_label, 2, 0)
        self.grid.addWidget(self.inputNumber, 2, 1)
        self.grid.addWidget(self.ok, 3, 0)

        self.window = QtWidgets.QWidget()
        self.setWindowTitle("Video calculator")
        self.setStyleSheet('QMainWindow{background-color: darkgray;}')
        self.window.setGeometry(450, 90, 600, 50)
        self.window.setLayout(self.grid)
        self.window.show()

    def on_select(self):

        if self.combo.currentText() == "Subtraction":

            self.load1.setEnabled(True)

        elif self.combo.currentText() == "ADD":

            self.load1.setEnabled(True)

        elif self.combo.currentText() == "Divide":

            self.load1.setEnabled(True)

        elif self.combo.currentText() == "Multiplication":

            self.load1.setEnabled(True)

    def updata_input_video(self, data_in, label):
        if label == 0:
            self.original_video_1 = data_in[0]
            self.title_0 = data_in[1]
            self.file_name_0 = data_in[2]
        elif label == 1:
            self.original_video_2 = data_in[0]
            self.title_1 = data_in[1]
            self.file_name_1 = data_in[2]

    def read_data1(self):
        reading = Reading()
        reading.update_output.connect(partial(self.updata_input_video, label=0))
        reading.read_video()

        self.inputFileLineEdit1.setText(self.file_name_0)

    def display_result(self, input_video, headr_name):
        visualization_ = Visulization_localization()
        visualization_.new_display(input_video, input_video, object=None, title=headr_name)

    def do_update(self):

        if self.ok.clicked:
            flag_read_number = self.get_values()
            if flag_read_number:
                if self.combo.currentText() == "Subtraction":
                    if self.original_video_1 is not None:
                        self.out_video = np.subtract(self.original_video_1, self.constant_number)
                        self.display_result(self.out_video, "Subtraction")
                        self.update_output.emit([self.out_video, self.combo.currentText(), None])

                    else:
                        self.msg_box = QtWidgets.QMessageBox()
                        self.msg_box.setWindowTitle("Warning!")
                        self.msg_box.setText("Please import videos!")
                        self.msg_box.exec_()

                elif self.combo.currentText() == "ADD":
                    if self.original_video_1 is not None:
                        self.out_video = np.add(self.original_video_1, self.constant_number)
                        self.display_result(self.out_video, "ADD")
                        self.update_output.emit([self.out_video, self.combo.currentText(), None])

                    else:
                        self.msg_box = QtWidgets.QMessageBox()
                        self.msg_box.setWindowTitle("Warning!")
                        self.msg_box.setText("Please import videos!")
                        self.msg_box.exec_()

                elif self.combo.currentText() == "Divide":
                    if self.original_video_1 is not None:
                        self.out_video = np.divide(self.original_video_1, self.constant_number)
                        self.display_result(self.out_video, "Divide")
                        self.update_output.emit([self.out_video, self.combo.currentText(), None])

                    else:
                        self.msg_box = QtWidgets.QMessageBox()
                        self.msg_box.setWindowTitle("Warning!")
                        self.msg_box.setText("Please import videos!")
                        self.msg_box.exec_()

                elif self.combo.currentText() == "Multiplication":

                    if self.original_video_1 is not None:
                        self.out_video = np.multiply(self.original_video_1, self.constant_number)
                        self.display_result(self.out_video, "Multiplication")
                        self.update_output.emit([self.out_video, self.combo.currentText(), None])
                    else:
                        self.msg_box = QtWidgets.QMessageBox()
                        self.msg_box.setWindowTitle("Warning!")
                        self.msg_box.setText("Please import video!")
                        self.msg_box.exec_()

            else:
                self.msg_box = QtWidgets.QMessageBox()
                self.msg_box.setWindowTitle("Warning!")
                self.msg_box.setText("Please fill the number text box!")
                self.msg_box.exec_()

    def get_values(self):
        try:
            self.constant_number = float(self.inputNumber.text())
            flag_read_number = True
        except:
            flag_read_number = False
        return flag_read_number

    def closeEvent(self, event):
        QtCore.QCoreApplication.instance().quit()
        print("closing PlaySetting")




import pytest
import pathlib
import os
import wget
import zipfile
import numpy as np

from piscat.InputOutput import video_reader

DATA_PATH = pathlib.Path(__file__).parent.parent / 'data'

@pytest.fixture
def control_video_path():

    video_path = DATA_PATH / 'Control' / 'control_4999_128_128_uint16_2.33FPS.raw'
    zip_file = DATA_PATH / 'Control.zip'
    if not os.path.exists(video_path):
        zip_file = wget.download('https://owncloud.gwdg.de/index.php/s/tzRZ7ytBd1weNDl/download')
        zip = zipfile.ZipFile(zip_file)
        zip.extractall(DATA_PATH)
        zip.close()
    if os.path.exists(zip_file):
        os.remove(zip_file)

    return video_path


@pytest.fixture()
def control_video(control_video_path):
    return video_reader(file_name=control_video_path, type='binary', img_width=128, img_height=128,
                                        image_type=np.dtype('<u2'), s_frame=0, e_frame=-1)



from __future__ import print_function

import pandas as pd
import json
import os
import sys


class CameraParameters:

    def __init__(self, name, quantum_efficiency=0.3, electron_well_depth=180e3, max_electron_well_depth=200e3, bit_depth=12, pixelSize=0.66):
        """
        Based on the camera features, this class generates a JSON file.
        This JSON was used by other functions and methods to set certain parameters, such as pixel size.

        Parameters
        ----------
        name: str
            Name of camera.

        quantum_efficiency: float
            Quantum efficiency of camera.

        electron_well_depth: float
             Electron well depth of camera.

        max_electron_well_depth: float
            Maximum electron well depth of camera

        bit_depth: float
            Bit depth of the camera.

        pixelSize: float
            Pixel size of camera.
        """
        try:
            self.read_camera_setting(name=name)
        except FileNotFoundError:
            self.quantum_efficiency = quantum_efficiency
            self.electron_well_depth = electron_well_depth
            self.max_electron_well_depth = max_electron_well_depth
            self.bit_depth = bit_depth
            self.pixelSize = pixelSize

            self.setting_dic = {'quantum_efficiency': [self.quantum_efficiency],
                                'electron_well_depth': [self.electron_well_depth],
                                'max_electron_well_depth': [self.max_electron_well_depth],
                                'bit_depth': [self.bit_depth],
                                'pixelSize': [self.pixelSize]}
            self.save_camera_setting(name)

    def save_camera_setting(self, name='Photonfocus.json'):

        here = os.path.dirname(os.getcwd())
        subdir = 'piscat_configuration'

        try:
            dr_mk = os.path.join(here, subdir)
            os.mkdir(dr_mk)
            print("Directory ", subdir, " Created ")
        except FileExistsError:
            print("Directory ", subdir, " already exists")

        filepath = os.path.join(here, subdir, name)
        df_configfile = pd.DataFrame(data=self.setting_dic)
        df_configfile.to_json(filepath)

    def read_camera_setting(self, name='Photonfocus.json'):
        subdir = 'piscat_configuration'
        here = os.path.dirname(os.getcwd())
        filepath = os.path.join(here, subdir, name)

        with open(filepath) as json_file:
            cam_setting = json.load(json_file)

        self.quantum_efficiency = cam_setting['quantum_efficiency']['0']
        self.electron_well_depth = cam_setting['electron_well_depth']['0']
        self.max_electron_well_depth = cam_setting['max_electron_well_depth']['0']
        self.bit_depth = cam_setting['bit_depth']['0']
        self.pixelSize = cam_setting['pixelSize']['0']









from piscat.Trajectory import particle_linking, temporal_filtering
from piscat.Visualization import plot, plot_histogram

from PySide2 import QtGui, QtCore, QtWidgets


class Tracking_GUI(QtWidgets.QWidget):
    update_tracking = QtCore.Signal(object)
    update_trajectories = QtCore.Signal(object)
    output_setting_Tab_tracking = QtCore.Signal(object)
    output_number_Particels_tracking = QtCore.Signal(object)

    def __init__(self, batch_size, parent=None):
        super(Tracking_GUI, self).__init__(parent)

        self.batch_size = batch_size

        self.input_video = None
        self.link_df_PSFS = None
        self.df_psfs = None
        self.his_all_particles = None
        self.memory = None
        self.search_range = None
        self.temporal_length = None
        self.df_PSFs_link = None
        self.pixel_size = 1
        self.axisScale = 'nm'

        self.setting_tracking = {}
        self.PSFs_Particels_num = {}

        self.empty_value_box_flag = False
        self.empty_value_optional_box_flag_1 = False
        self.empty_value_optional_box_flag_2 = False

        self.resize(300, 300)
        self.setWindowTitle('PSFs Tracking')

        self.btn_hist_plot = QtWidgets.QPushButton("Plot Len histogram")
        self.btn_hist_plot.setFixedWidth(150)
        self.btn_hist_plot.setFixedHeight(30)
        self.btn_hist_plot.clicked.connect(self.plot_len_hist)

        self.btn_linking = QtWidgets.QPushButton("Update linking")
        self.btn_linking.setFixedWidth(120)
        self.btn_linking.setFixedHeight(30)
        self.btn_linking.clicked.connect(self.do_update)

        self.btn_filterLinking = QtWidgets.QPushButton("Temporal filter")
        self.btn_filterLinking.setFixedWidth(120)
        self.btn_filterLinking.setFixedHeight(30)
        self.btn_filterLinking.clicked.connect(self.do_update_tfilter)

        self.btn_plot2Dlocalization = QtWidgets.QPushButton("Plot 2D localization")
        self.btn_plot2Dlocalization.setFixedWidth(120)
        self.btn_plot2Dlocalization.setFixedHeight(20)
        self.btn_plot2Dlocalization.clicked.connect(self.do_plot2D)

        self.line_memory = QtWidgets.QLineEdit()
        self.line_memory.setPlaceholderText("Memory")
        self.line_memory_label = QtWidgets.QLabel("Memory (#frames):")

        self.line_search_range = QtWidgets.QLineEdit()
        self.line_search_range.setPlaceholderText("Move")
        self.line_searchRange_label = QtWidgets.QLabel("Neighborhood size(px):")

        self.line_min_V_shape = QtWidgets.QLineEdit()
        self.line_min_V_shape.setPlaceholderText("V_shape_width")
        self.line_min_V_shape_label = QtWidgets.QLabel("V_shape_width (#frames):")

        self.grid = QtWidgets.QGridLayout()
        self.grid.addWidget(self.createFirstExclusiveGroup(), 0, 0)
        self.grid.addWidget(self.createSecondExclusiveGroup(), 1, 0)

        self.setLayout(self.grid)

    def __del__(self):
        print('Destructor called, Employee deleted.')

    def createFirstExclusiveGroup(self):

        groupBox = QtWidgets.QGroupBox("Linking")
        self.checkbox_sorting_based_lenght = QtWidgets.QCheckBox("Sorting", self)


        self.grid_linking = QtWidgets.QGridLayout()

        self.grid_linking.addWidget(self.line_memory_label, 0, 0)
        self.grid_linking.addWidget(self.line_memory, 0, 1)

        self.grid_linking.addWidget(self.line_searchRange_label, 1, 0)
        self.grid_linking.addWidget(self.line_search_range, 1, 1)

        self.grid_linking.addWidget(self.line_min_V_shape_label, 2, 0)
        self.grid_linking.addWidget(self.line_min_V_shape, 2, 1)
        self.grid_linking.addWidget(self.btn_linking, 3, 0)
        self.grid_linking.addWidget(self.btn_hist_plot, 3, 1)
        self.grid_linking.addWidget(self.btn_filterLinking, 3, 2)

        self.grid_linking.addWidget(self.checkbox_sorting_based_lenght, 3, 3)

        groupBox.setLayout(self.grid_linking)
        return groupBox

    def createSecondExclusiveGroup(self):

        self.checkbox_display_label = QtWidgets.QCheckBox("Display label of particles", self)

        self.line_pixel_size = QtWidgets.QLineEdit()
        self.line_pixel_size.setPlaceholderText("Pixel Size")
        self.line_pixel_size_label = QtWidgets.QLabel("Pixel size:")

        self.line_axis_scale = QtWidgets.QLineEdit()
        self.line_axis_scale.setPlaceholderText("Scale 'nm' ")
        self.line_axis_scale_label = QtWidgets.QLabel("Axis scale")

        groupBox = QtWidgets.QGroupBox("Optional")

        self.grid1 = QtWidgets.QGridLayout()
        self.grid1.addWidget(self.checkbox_display_label, 0, 0)
        self.grid1.addWidget(self.line_pixel_size_label, 1, 0)
        self.grid1.addWidget(self.line_pixel_size, 1, 1)
        self.grid1.addWidget(self.line_axis_scale_label, 2, 0)
        self.grid1.addWidget(self.line_axis_scale, 2, 1)
        self.grid1.addWidget(self.btn_plot2Dlocalization, 3, 1)

        groupBox.setLayout(self.grid1)
        return groupBox

    @QtCore.Slot()
    def update_in_data(self, data_in):
        self.input_video = data_in[0]
        self.df_psfs = data_in[1]

    @QtCore.Slot()
    def update_batchSize(self, batchSize):
        self.batch_size = batchSize

    @QtCore.Slot()
    def do_update(self):

        if self.df_psfs is None:
            self.msg_box2 = QtWidgets.QMessageBox()
            self.msg_box2.setWindowTitle("Warning!")
            self.msg_box2.setText("Please localized PSFs!")
            self.msg_box2.exec_()

        else:

            self.get_values_1()
            if self.empty_value_box_flag:
                if self.df_psfs.shape[0] > 1:
                    linking_ = particle_linking.Linking()
                    df_PSFs_link = linking_.create_link(psf_position=self.df_psfs,
                                                        search_range=self.search_range,
                                                        memory=self.memory)
                    self.df_PSFs_link = df_PSFs_link

                    self.his_all_particles = df_PSFs_link['particle'].value_counts()
                    plot_histogram(self.his_all_particles, 'Lengths of linking')
                    self.PSFs_Particels_num['Total_number_Particles'] = linking_.trajectory_counter(df_PSFs_link)
                else:
                    self.msg_box3 = QtWidgets.QMessageBox()
                    self.msg_box3.setWindowTitle("Warning!")
                    self.msg_box3.setText("No PSFs found for linking!")
                    self.msg_box3.exec_()

    def plot_len_hist(self):
        if self.his_all_particles is not None:
            plot_histogram(self.his_all_particles, 'Lengths of linking')
        else:
            self.msg_box3 = QtWidgets.QMessageBox()
            self.msg_box3.setWindowTitle("Warning!")
            self.msg_box3.setText("Please update linking!")
            self.msg_box3.exec_()

    def do_update_tfilter(self):
        if self.df_PSFs_link is None:
            self.msg_box2 = QtWidgets.QMessageBox()
            self.msg_box2.setWindowTitle("Warning!")
            self.msg_box2.setText("Please linking PSFs!")
            self.msg_box2.exec_()

        else:
            self.get_values_2()
            if self.empty_value_box_flag:
                if self.df_PSFs_link.shape[0] > 1:
                    t_filters = temporal_filtering.TemporalFilter(video=self.input_video,
                                                                  batchSize=self.batch_size)

                    all_trajectories, df_PSFs_t_filter, his_all_particles = t_filters.v_trajectory(df_PSFs=self.df_PSFs_link,
                                                                                                   threshold=self.temporal_length)

                    linking_ = particle_linking.Linking()
                    self.PSFs_Particels_num['#Particles_after_V_shapeFilter'] = linking_.trajectory_counter(df_PSFs_t_filter)

                    if self.checkbox_sorting_based_lenght.isChecked():
                        df_PSFs_t_filter = linking_.sorting_linking(df_PSFs=df_PSFs_t_filter)

                    self.link_df_PSFS = df_PSFs_t_filter

                    self.setting_tracking['search_range'] = self.search_range
                    self.setting_tracking['Memory'] = self.memory
                    self.setting_tracking['minimum_temporal_length'] = self.temporal_length

                    self.update_tracking.emit(self.link_df_PSFS)
                    self.update_trajectories.emit(all_trajectories)
                    self.output_setting_Tab_tracking.emit(self.setting_tracking)
                    self.output_number_Particels_tracking.emit(self.PSFs_Particels_num)
            else:
                self.msg_box3 = QtWidgets.QMessageBox()
                self.msg_box3.setWindowTitle("Warning!")
                self.msg_box3.setText("No PSFs found for temporal filtering!")
                self.msg_box3.exec_()

    def do_plot2D(self):

        if self.link_df_PSFS is None:
            self.msg_box3 = QtWidgets.QMessageBox()
            self.msg_box3.setWindowTitle("Warning!")
            self.msg_box3.setText("Please update the linking section!")
            self.msg_box3.exec_()
        else:

            try:
                self.pixel_size = int(self.line_pixel_size.text())
            except:
                self.msg_box3 = QtWidgets.QMessageBox()
                self.msg_box3.setWindowTitle("Warning!")
                self.msg_box3.setText("Pixel size set as default (1 nm)!")
                self.msg_box3.exec_()

                self.pixel_size = 1

            try:
                self.axisScale = str(self.line_axis_scale.text())
            except:
                self.msg_box3 = QtWidgets.QMessageBox()
                self.msg_box3.setWindowTitle("Warning!")
                self.msg_box3.setText("axis scale set as default (nm)!")
                self.msg_box3.exec_()

                self.axisScale = '(nm)'

            if self.checkbox_display_label.isChecked():
                label = True
            else:
                label = False

            plot.plot2df(self.link_df_PSFS, pixel_size=self.pixel_size, scale=self.axisScale, title='', flag_label=label)

    def get_values_1(self):
        try:
            self.memory = int(self.line_memory.text())
            self.search_range = int(self.line_search_range.text())

            self.empty_value_box_flag = True

        except:
            self.msg_box3 = QtWidgets.QMessageBox()
            self.msg_box3.setWindowTitle("Warning!")
            self.msg_box3.setText("Please filled all parameters!")
            self.msg_box3.exec_()

            self.empty_value_box_flag = False

    def get_values_2(self):
        try:
            self.temporal_length = int(self.line_min_V_shape.text())

            self.empty_value_box_flag = True

        except:
            self.msg_box3 = QtWidgets.QMessageBox()
            self.msg_box3.setWindowTitle("Warning!")
            self.msg_box3.setText("Please filled all parameters!")
            self.msg_box3.exec_()

            self.empty_value_box_flag = False

    def closeEvent(self, event):
        QtCore.QCoreApplication.instance().quit()



from .filtering import FFT2D
from .filtering import Filters
from .filtering import RadialVarianceTransform
from .filtering import FastRadialSymmetryTransform
from .filtering import *
from .normalization import Normalization
from .patch_genrator import ImagePatching



from __future__ import print_function

from scipy import optimize
from matplotlib.patches import Ellipse

import numpy as np
import math
import warnings
import matplotlib.pyplot as plt

warnings.filterwarnings('ignore')


def gaussian_2d(xy_mesh, amp, xc, yc, sigma_x, sigma_y, b):
    (x, y) = xy_mesh

    # make the 2D Gaussian matrix
    gauss = b + amp * np.exp(-(((x - xc) ** 2 / ((sigma_x ** 2))) + ((y - yc) ** 2 / ((sigma_y ** 2)))))

    # flatten the 2D Gaussian down to 1D
    return np.ravel(gauss)


def fit_2D_Gaussian_varAmp(image, sigma_x, sigma_y, display_flag=False):
    """
    This function uses non-linear squares to fit 2D Gaussian.

    Parameters
    ----------
    image: NDArray
        2D numpy array, image.

    sigma_x: float
        It is initial value for sigma X.

    sigma_y: float
        It is initial value for sigma y.

    display_flag: bool
        This flag is used to display the result of fitting for each PSF.

    Returns
    -------
    @return: (list)
            [sigma_ratio, fit_params, fit_errors]
    """
    x = np.linspace(0, image.shape[0] - 1, image.shape[0], dtype=np.int)
    y = np.linspace(0, image.shape[1] - 1, image.shape[1], dtype=np.int)
    xy_mesh = np.meshgrid(y, x)

    data = np.transpose(image)
    data = np.reshape(data, (data.shape[0] * data.shape[1], 1))
    try:
        if (np.median(data) - np.min(data)) > (np.max(data) - np.median(data)):
            i_amp = - (np.median(data) - np.min(data))
        else:
            i_amp = np.max(data) - np.median(data)
    except ValueError:
        i_amp = 1

    amp = i_amp
    b = np.median(data)
    xc = int(image.shape[1] / 2)
    yc = int(image.shape[0] / 2)
    sigma_x = sigma_x
    sigma_y = sigma_y
    guess_vals = [amp, xc, yc, sigma_x, sigma_y, b]
    try:
        fit_params, cov_mat = optimize.curve_fit(gaussian_2d, xy_mesh, np.ravel(image), p0=guess_vals, maxfev=5000)
        fit_errors = np.sqrt(np.diag(cov_mat))

        fit_residual = image - gaussian_2d(xy_mesh, *fit_params).reshape(np.outer(x, y).shape)
        fit_Rsquared = 1 - np.var(fit_residual) / np.var(image)

        if display_flag is True:

            print('Fit R-squared:', fit_Rsquared, '\n')
            print('Fit Amplitude:', fit_params[0], '\u00b1', fit_errors[0])
            print('Fit X-Center: ', fit_params[1], '\u00b1', fit_errors[1])
            print('Fit Y-Center: ', fit_params[2], '\u00b1', fit_errors[2])
            print('Fit X-Sigma:  ', fit_params[3], '\u00b1', fit_errors[3])
            print('Fit Y-Sigma:  ', fit_params[4], '\u00b1', fit_errors[4])
            print('Fit Bias:  ', fit_params[5], '\u00b1', fit_errors[5])

            plt.figure()
            plt.imshow(image)
            # draw the ellipse
            ax = plt.gca()
            ax.add_patch(Ellipse((fit_params[2], fit_params[1]), width=math.sqrt(2) * fit_params[3],
                                 height=math.sqrt(2) * fit_params[4],
                                 edgecolor='white',
                                 facecolor='none',
                                 linewidth=5))

            if abs(fit_params[3]) < abs(fit_params[4]):
                sigma_ratio = abs(fit_params[3] / fit_params[4])
                print('X-Sigma/Y-Sigma:  ', fit_params[3] / fit_params[4])
            else:
                sigma_ratio = abs(fit_params[4] / fit_params[3])
                print('Y-Sigma/X-Sigma:  ', fit_params[4] / fit_params[3])

        else:
            if abs(fit_params[3]) < abs(fit_params[4]):
                sigma_ratio = abs(fit_params[3] / fit_params[4])
            else:
                sigma_ratio = abs(fit_params[4] / fit_params[3])
    except:
        sigma_ratio = 1
        fit_params = None
        fit_errors = None
    return [sigma_ratio, fit_params, fit_errors]



from piscat.GUI.CPU_Configurations import Import_configuration
from piscat.InputOutput.cpu_configurations import CPUConfigurations
from PySide2 import QtWidgets
from PySide2 import QtCore


class CPU_setting_wrapper(QtWidgets.QMainWindow):

    def __init__(self):
        super(CPU_setting_wrapper, self).__init__()
        self.change_cpu_setting()

    def change_cpu_setting(self):
        print('\nChange CPU setting-->', end='')
        self.cpu_setting = Import_configuration.CPU_Setting()
        self.cpu_setting.update_CPU_Setting.connect(self.save_CPU_setting)

    @QtCore.Slot()
    def save_CPU_setting(self):
        if self.cpu_setting.parallel_active is True:

            setting_dic = {'n_jobs': [self.cpu_setting.n_jobs], 'backend': [self.cpu_setting.backend], 'verbose': [self.cpu_setting.verbose],
                           'parallel_active': [self.cpu_setting.parallel_active],
                           'threshold_for_parallel_run': [self.cpu_setting.threshold_for_parallel_run]}

            cpu_setting = CPUConfigurations()
            cpu_setting.save_cpu_setting(setting_dic)
        else:
            setting_dic = {'n_jobs': [1], 'backend': ['None'],
                           'verbose': [0],
                           'parallel_active': [self.cpu_setting.parallel_active],
                           'threshold_for_parallel_run': [None]}
            cpu_setting = CPUConfigurations()
            cpu_setting.save_cpu_setting(setting_dic)
        print("Done!")


from PySide2 import QtGui, QtCore, QtWidgets


class DRA(QtWidgets.QMainWindow):

    signal_finish = QtCore.Signal(bool)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.batch_size = None

        self.flag_display = False
        self.flag_power_normalization = False

        self.window = QtWidgets.QWidget()

        self.le1 = QtWidgets.QLineEdit()
        self.le1.setPlaceholderText('radius')

        self.ok = QtWidgets.QPushButton("Ok")
        self.ok.setAutoDefault(False)
        self.ok.clicked.connect(self.do_update)

        self.checkbox_display = QtWidgets.QCheckBox("Display", self)
        self.checkbox_power_normalization = QtWidgets.QCheckBox("Laser power normalization", self)

        self.grid = QtWidgets.QGridLayout()
        self.grid.addWidget(self.le1, 0, 0)
        self.grid.addWidget(self.checkbox_display, 0, 1)
        self.grid.addWidget(self.checkbox_power_normalization, 1, 1)
        self.grid.addWidget(self.ok, 2, 0)

        self.window.setWindowTitle("Rolling Average")
        self.setStyleSheet('QMainWindow{background-color: darkgray;}')
        self.window.setLayout(self.grid)
        self.window.show()

    def do_update(self):
        if self.ok.clicked:
            self.get_values()
            if self.checkbox_display.isChecked():
                self.flag_display = True
            else:
                self.flag_display = False

            if self.checkbox_power_normalization.isChecked():
                self.flag_power_normalization = True

            if self.batch_size != '':
                self.batch_size = int(self.batch_size)
                self.signal_finish.emit(True)

            else:
                self.msg_box = QtWidgets.QMessageBox()
                self.msg_box.setWindowTitle("Warning!")
                self.msg_box.setText("Please insert batch size!")
                self.msg_box.exec_()

    def get_values(self):
        self.batch_size = self.le1.text()

    def closeEvent(self, event):
        QtCore.QCoreApplication.instance().quit()




import sys
import os
import multiprocessing
import warnings
import datetime
import time
import pkg_resources

from piscat.GUI.InputOutput import Reading
from piscat.GUI.CPU_Configurations import CPU_setting_wrapper
from piscat.GUI.Proccessing import FFT2D_GUI_wrapper
from piscat.GUI.VideoAnalysis import Analysis
from piscat.GUI.VideoAnalysis import AnalysisConstant
from piscat.GUI.ProgressBar.fun_progressBar import ProgressBar
from piscat.GUI.Projects.Protein import ProteinTabs
from piscat.GUI.Projects.Protein import Noise_Floor
from piscat.GUI.Memory.video_in_memory import VideoInMemory
from piscat.GUI.Visualization.fun_display_localization import Visulization_localization
from piscat import version

from PySide2 import QtGui
from PySide2 import QtCore
from PySide2 import QtWidgets
from PySide2.QtCore import *
from PySide2.QtWidgets import QMenuBar
from functools import partial

warnings.filterwarnings('ignore')


class PiSCAT_GUI(QtWidgets.QMainWindow):

    progressChanged = QtCore.Signal(int)

    def __init__(self):
        super(PiSCAT_GUI, self).__init__()

        self.threadpool = QThreadPool()
        self.original_video = None
        self.dra_video = None

        self.FFT2D_GUI_wrapper = FFT2D_GUI_wrapper()

        self.initUI()

    def initUI(self):

        # defining the main window.
        window_icon = pkg_resources.resource_filename('piscat.GUI.icons', 'mpl.png')
        main_icon = QtGui.QIcon(window_icon)
        self.setWindowIcon(main_icon)
        self.setWindowTitle("PiSCAT")
        self.setStyleSheet('QMainWindow{background-color: darkgray;}')
        self.setGeometry(90, 90, 400, 1000)
        self.setFixedSize(340, 600)

        # defining the main frame as the central widget and give it a vertical layout.
        self.main_frame = QtWidgets.QFrame()
        self.setCentralWidget(self.main_frame)
        self.main_frame_layout = QtWidgets.QVBoxLayout()
        self.main_frame.setLayout(self.main_frame_layout)

        # ------------------------- Adding the main Logo ---------------------------------------------
        label_logo = QtWidgets.QLabel(self)
        logo_piscat_path = pkg_resources.resource_filename('piscat.GUI.icons', 'PiSCAT_logo_bg.png')
        loader_logo = QtGui.QPixmap(logo_piscat_path)
        loader_logo = loader_logo.scaled(320, 100)
        label_logo.setPixmap(loader_logo)
        self.main_frame_layout.addWidget(label_logo)

        label = QtWidgets.QLabel(self)
        movie_path = pkg_resources.resource_filename('piscat.GUI.icons', 'movie.gif')
        loader_gif = QtGui.QMovie(movie_path)
        self.resize(loader_gif.currentPixmap().width(), loader_gif.currentPixmap().height())
        loader_gif.start()
        label.setMovie(loader_gif)
        self.main_frame_layout.addWidget(label)

        # ------------------defining the toolbox GroupBox and add it to the main frame.---------------
        self.toolbox_GB = QtWidgets.QGroupBox("Information")
        self.toolbox_GB.setFixedHeight(200)
        self.toolbox_GB_layout = QtWidgets.QVBoxLayout()
        self.toolbox_GB.setLayout(self.toolbox_GB_layout)
        self.main_frame_layout.addWidget(self.toolbox_GB)

        self.textEdit = QtWidgets.QTextEdit()
        self.textEdit.setReadOnly(True)
        self.textEdit.verticalScrollBar().minimum()
        # textEdit.setTextBackgroundColor(QtGui.QColor(140,140,0))
        self.toolbox_GB_layout.addWidget(self.textEdit)

        # adding different tab of "File" in menuBar.
        menubar = self.menuBar()

        #--------File------
        fileMenu = menubar.addMenu('&File')

        open_file = QtWidgets.QAction(QtGui.QIcon('folder.ico'), 'Open', self)
        # open_file.setShortcut('Ctrl+S')
        open_file.setStatusTip('Open new File')
        fileMenu.addAction(open_file)
        self.connect(open_file, QtCore.SIGNAL('triggered()'), self.video_loading_wrapper)

        imp = fileMenu.addMenu("Import")
        open_im2vid = imp.addAction("Image Sequence to video")
        self.connect(open_im2vid, QtCore.SIGNAL('triggered()'), self.image2video_loading_wrapper)

        load_python_script = imp.addAction("Run Python script")
        self.connect(load_python_script, QtCore.SIGNAL('triggered()'), Reading().run_py_script)
        # --------File------

        # --------Setting--------
        setting_menu = menubar.addMenu('&Setting')

        setting_cpu = QtWidgets.QAction(QtGui.QIcon('folder.ico'), 'CPU_Configurations', self)
        # setting_cpu.setShortcut('Ctrl+O')
        setting_cpu.setStatusTip('Open new File')
        setting_menu.addAction(setting_cpu)
        self.connect(setting_cpu, QtCore.SIGNAL('triggered()'), self.cpu_setting)
        # --------Setting--------

        # --------View --------
        self.list_available_video = {}
        view_menu = menubar.addMenu('&View')

        self.video_in_memory_flag = {'original_video': False, 'DRA_video': False, 'Protein_Tracking': False, 'GUV3D': False,
                                     'arc_cos': False, 'PSF Clustering': False, 'Template_matching': False}

        Display_video_in_memory = QtWidgets.QAction(QtGui.QIcon('folder.ico'), 'Load videos', self)
        # open_file.setShortcut('Ctrl+S')
        Display_video_in_memory.setStatusTip('Display')
        view_menu.addAction(Display_video_in_memory)
        self.connect(Display_video_in_memory, QtCore.SIGNAL('triggered()'), self.available_video_in_memory)
        # --------View --------

        # --------Process--------
        process_menu = menubar.addMenu('&Process')

        spectrum = QtWidgets.QAction(QtGui.QIcon('folder.ico'), 'FFT', self)
        # open_file.setShortcut('Ctrl+S')
        spectrum.setStatusTip('FFT')
        process_menu.addAction(spectrum)
        self.connect(spectrum, QtCore.SIGNAL('triggered()'), self.spectrum_init)

        image_calculator = QtWidgets.QAction(QtGui.QIcon('folder.ico'), 'Image calculator', self)
        # open_file.setShortcut('Ctrl+S')
        image_calculator.setStatusTip('Image calculator')
        process_menu.addAction(image_calculator)
        self.connect(image_calculator, QtCore.SIGNAL('triggered()'), self.analysis_wrapper)

        image_calculator_2 = QtWidgets.QAction(QtGui.QIcon('folder.ico'), 'Image calculator constant number', self)
        # open_file.setShortcut('Ctrl+S')
        image_calculator_2.setStatusTip('Image calculator constant number')
        process_menu.addAction(image_calculator_2)
        self.connect(image_calculator_2, QtCore.SIGNAL('triggered()'), self.analysis_wrapper_2)
        # --------Process--------

        # --------Analyze.--------
        analyze_menu = menubar.addMenu('&Analyze')

        noise_floor = QtWidgets.QAction('Noise Floor', self)
        # open_file.setShortcut('Ctrl+R')
        noise_floor.setStatusTip('Noise Floor.')
        self.connect(noise_floor, QtCore.SIGNAL('triggered()'), self.noise_floor)

        protein_track = QtWidgets.QAction("iSCAT Protein", self)
        protein_track.setStatusTip("iSCAT Protein")
        self.connect(protein_track, QtCore.SIGNAL('triggered()'), self.protein_projects)

        analyze_menu.addAction(noise_floor)
        analyze_menu.addAction(protein_track)
        # --------Analyze.--------

        # --------Information Box--------
        save_txt = QtWidgets.QAction('Report', self)
        # save_txt.setShortcut('Ctrl+Q')
        save_txt.setStatusTip('save history')
        save_txt.triggered.connect(self.save_text_history)
        toolbar_save = self.addToolBar('Save')
        toolbar_save.addAction(save_txt)

        clear_txt = QtWidgets.QAction('Clear', self)
        # exitAction.setShortcut('Ctrl+Q')
        clear_txt.setStatusTip('clear Information Box')
        clear_txt.triggered.connect(self.clear_plain_text)

        toolbar_clear = self.addToolBar('Clear')
        toolbar_clear.addAction(clear_txt)

        exitAction = QtWidgets.QAction('Exit', self)
        # exitAction.setShortcut('Ctrl+Q')
        exitAction.setStatusTip('Exit application')
        exitAction.triggered.connect(self.closeEvent)
        fileMenu.addAction(exitAction)
        toolbar = self.addToolBar('Exit')
        toolbar.addAction(exitAction)
        # --------Information Box--------

        self.statusbar = self.statusBar()
        self.progressBar = QtWidgets.QProgressBar(self)
        self.statusBar().addPermanentWidget(self.progressBar)
        self.progressBar.setGeometry(30, 40, 200, 25)
        self.progressBar.setValue(0)
        self.update_progressBar = ProgressBar(self.progressBar)

        self.show()

    def __del__(self):
        print('Destructor called, Employee deleted.')

    def closeEvent(self, event):
        QtCore.QCoreApplication.instance().quit()
        print("closing PlaySetting")

    @QtCore.Slot()
    def set_new_text(self, input_text):
        if type(input_text) is str:
            self.textEdit.append(input_text)

    @QtCore.Slot()
    def set_plain_text(self, input_text):
        if type(input_text) is str:
            self.textEdit.insertPlainText(input_text)

    def clear_plain_text(self):
        self.textEdit.clear()

    def save_text_history(self):
        self.file_path = False
        self.file_path, _ = QtWidgets.QFileDialog.getSaveFileName(self, "project_report",
                                                              QtCore.QDir.currentPath())
        timestr = time.strftime("%Y%m%d-%H%M%S")

        with open(self.file_path + '_' + timestr +'.txt', 'w') as outfile:
            outfile.write(str(self.textEdit.toPlainText()))

    def spectrum_init(self):
        try:
            self.FFT2D_GUI_wrapper.spectrum_input(self.reading.original_video)
            self.FFT2D_GUI_wrapper.spectrum()

        except AttributeError:
            self.msg_box = QtWidgets.QMessageBox()
            self.msg_box.setWindowTitle("Warning!")
            self.msg_box.setText("input video does not find!")
            self.msg_box.exec_()

    def analysis_wrapper(self):
        self.analysis_videos = Analysis()
        self.analysis_videos.update_output.connect(partial(self.updata_input_video, label='Video_analysis'))

    def analysis_wrapper_2(self):
        self.analysis_constant = AnalysisConstant()
        self.analysis_constant.update_output.connect(partial(self.updata_input_video, label='Video_analysis_constant'))

    def cpu_setting(self):
        self.processor_setting = CPU_setting_wrapper()

    def available_video_in_memory(self):
        self.vid_in_memory = VideoInMemory(self.list_available_video)
        self.vid_in_memory.display_trigger.connect(self.video_display)

    def video_loading_wrapper(self):
        self.reading = Reading()
        self.reading.update_output.connect(partial(self.updata_input_video, label='loading'))
        self.reading.read_video()

    def image2video_loading_wrapper(self):
        self.reading = Reading()
        self.reading.update_output.connect(partial(self.updata_input_video, label='loading'))
        self.reading.im2video()

    def noise_floor(self):
        self.noise_floor_ = Noise_Floor(video=self.original_video)

    def protein_projects(self):
        self.protein_gui = ProteinTabs(video_in=self.original_video, batch_size=None, object_update_progressBar=self.update_progressBar)
        self.protein_gui.new_update_DRA_video.connect(partial(self.updata_input_video, label='DRA', flag_DRA=True))
        self.protein_gui.show()

    def updata_input_video(self, data_in, label, flag_DRA=False):

        if flag_DRA:
            self.dra_video = data_in[0]
            self.batch_size = data_in[3]
            self.list_available_video['DRA_video'] = True
            status_line_info = None
        else:
            status_line_info = data_in[3]
            self.original_video = data_in[0]
            self.list_available_video['original_video'] = True
        title = data_in[1]
        file_name = data_in[2]

        self.video_in_memory_flag[label] = True
        self.set_new_text('**** ' + str(datetime.datetime.now()) + ' ****')
        self.set_new_text(file_name)
        self.set_new_text(label + ' ' + title + " shape:" + str(self.original_video.shape))

        if status_line_info is not None and status_line_info['status_line_position'] != '':
            self.set_new_text('---Status line detected in ' + status_line_info['status_line_position'] + '---')
        elif status_line_info is not None and status_line_info['status_line_position'] == '':
            self.set_new_text('---Status line does not detect---')

    def video_display(self, data_in):
        if 'original_video' == data_in:
            self.visualization_ = Visulization_localization()
            self.visualization_.new_display(self.original_video, self.original_video, object=None, title='RAW', mask_status=False)

        elif 'DRA_video' == data_in:
            self.visualization_ = Visulization_localization()
            self.visualization_.new_display(self.dra_video, self.dra_video, object=None,
                                            title='DRA', mask_status=False)


def main():
    if sys.argv[0][-4:] == '.exe':
        setattr(sys, 'frozen', True)

    multiprocessing.freeze_support()
    app = QtWidgets.QApplication(sys.argv)
    mpl_icon = pkg_resources.resource_filename('piscat.GUI.icons', 'mpl.png')
    app.setWindowIcon(QtGui.QIcon(mpl_icon))

    ex = PiSCAT_GUI()

    print('Starting PiSCAT version %s' % version)
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()



import pickle

from ..Fixtures.shared import *
from piscat.Preproccessing import Normalization
import numpy as np


def test_preprocessing_normalization(control_video):

    video_pn, power_fluctuation = Normalization(video=control_video).power_normalized()

    expected = _load_expected(DATA_PATH / 'preprocessing_normalization.pck')

    _compare(video_pn, power_fluctuation, expected)


def test_preprocessing_normalization_parallel_vs_serial(control_video):

    vid_pn_parallel, fluct_parallel = Normalization(video=control_video).power_normalized(inter_flag_parallel_active=True)
    vid_pn_serial, fluct_serial = Normalization(video=control_video).power_normalized(inter_flag_parallel_active=False)

    np.testing.assert_array_almost_equal(vid_pn_serial, vid_pn_parallel)
    np.testing.assert_array_almost_equal(fluct_serial, fluct_parallel)


def _load_expected(file_name):
    with open(file_name, 'rb') as f:
        video_pn_first, video_pn_last, flucts = pickle.load(f)
    return video_pn_first, video_pn_last, flucts

def _compare(video_pn, power_fluctuation, expected):
    np.testing.assert_array_almost_equal(video_pn[0, ...], expected[0])
    np.testing.assert_array_almost_equal(video_pn[-1, ...], expected[1])
    np.testing.assert_array_almost_equal(power_fluctuation, expected[2])


from PySide2 import QtGui
from PySide2 import QtCore
from PySide2 import QtWidgets

from PySide2.QtCore import *
from PySide2.QtGui import *
from scipy.ndimage import median_filter

import numpy as np

from piscat.Localization.directional_intensity import DirectionalIntensity
from piscat.Preproccessing.normalization import Normalization
from piscat.Visualization.contrast_adjustment import ContrastAdjustment
from piscat.GUI.Visualization.updating_plots import UpdatingPlots


class SliceView(QtWidgets.QGraphicsView, QRunnable):

    photoClicked = QtCore.Signal(tuple)
    frameClicked = QtCore.Signal(int)
    pixmapClicked = QtCore.Signal(QtCore.QPoint)
    cursorMove = QtCore.Signal(object)
    annotation = QtCore.Signal()
    finished = Signal()

    x0 = 0
    y0 = 0
    x1 = 0
    y1 = 0
    flag_paint = False

    def __init__(self, input_video, video_original, stride=1, mask=False, *args, **kwargs):
        self.scene = QtWidgets.QGraphicsScene()
        super(SliceView, self).__init__(self.scene, *args, **kwargs)

        self.RAW_Video = video_original
        self.video_width = video_original.shape[1]
        self.video_height = video_original.shape[2]

        self.input_video = input_video
        self.con_adj = ContrastAdjustment(input_video)
        self.current_pixmap = None
        self.current_mask_pixmap = None
        self.maskArray = None
        self.mask_is_set = False
        self.livePaint_Flag = False
        self.medianFilterFlag = False
        self.updatePlot = UpdatingPlots()
        self.ori_X0 = 0
        self.ori_Y0 = 0
        self.ori_X1 = 0
        self.ori_Y1 = 0
        self.pixel_value = 0
        self.slice_num = 0

        self._zoom = 0
        self._empty = True

        self.annotation.connect(self.active_LivePaint)

        self.setBackgroundBrush(QtGui.QColor(0, 0, 0))
        self.setScene(self.scene)

        self.setTransformationAnchor(QtWidgets.QGraphicsView.AnchorUnderMouse)
        self.setResizeAnchor(QtWidgets.QGraphicsView.AnchorUnderMouse)

        self.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)

        if mask is True:
            self.initialze_mask()

    def __del__(self):
        print('Destructor called, Employee deleted.')

    def enterEvent(self, event):
        self.setMouseTracking(True)

    def leaveEvent(self, event):
        self.setMouseTracking(False)

    def pollCursor(self):
        pos = QtGui.QCursor.pos()
        if pos != self.cursor:
            self.cursor = pos
            self.cursorMove.emit(pos)

    def handleCursorMove(self, pos):
        print(pos)

    def hasPhoto(self):
        return not self._empty

    def toggleDragMode(self):
        if self.dragMode() == QtWidgets.QGraphicsView.ScrollHandDrag:
            self.setDragMode(QtWidgets.QGraphicsView.NoDrag)
        elif not self._photo.pixmap().isNull():
            self.setDragMode(QtWidgets.QGraphicsView.ScrollHandDrag)

    def mousePressEvent(self, event):

        if self._photo.isUnderMouse():

            sp = self.mapToScene(event.pos())
            lp = self._photo.mapFromScene(sp).toPoint()

            self.x0 = lp.x()
            self.y0 = lp.y()
            self.flag_paint = True

            self.ori_X0 = int((self.RAW_Video.shape[1] / self.pixmap_video.size().toTuple()[0]) * self.x0)
            self.ori_Y0 = int((self.RAW_Video.shape[2] / self.pixmap_video.size().toTuple()[1]) * self.y0)

            self.frameClicked.emit(self.slice_num)
            self.photoClicked.emit((self.ori_X0, self.ori_Y0))

        super(SliceView, self).mousePressEvent(event)

    def mouseReleaseEvent(self, event):
        self.flag_paint = False

    def mouseMoveEvent(self, event):
        if self.flag_paint:
            if self._photo.isUnderMouse():

                sp = self.mapToScene(event.pos())
                lp = self._photo.mapFromScene(sp).toPoint()

                self.x1 = lp.x()
                self.y1 = lp.y()

                self.ori_X1 = int((self.RAW_Video.shape[1] / self.pixmap_video.size().toTuple()[0]) * self.x1)
                self.ori_Y1 = int((self.RAW_Video.shape[2] / self.pixmap_video.size().toTuple()[1]) * self.y1)

                self.paint()

    def initialze_mask(self):

        if self.input_video.shape[1] >= self.input_video.shape[2]:

            self.mask_qimage = QtGui.QImage(QtCore.QSize(self.input_video.shape[2], self.input_video.shape[1]),
                                            QtGui.QImage.Format_ARGB32)

            self.mask_qimage.fill(QtGui.QColor(250, 240, 0, 0))
            self.mask_qimage_view = np.frombuffer(self.mask_qimage.bits(), dtype=np.uint8).reshape(
                self.input_video.shape[1],
                self.input_video.shape[2], 4)
        else:
            self.mask_qimage = QtGui.QImage(QtCore.QSize(self.input_video.shape[2], self.input_video.shape[1]),
                                            QtGui.QImage.Format_ARGB32)

            self.mask_qimage.fill(QtGui.QColor(250, 240, 0, 0))
            self.mask_qimage_view = np.frombuffer(self.mask_qimage.bits(), dtype=np.uint8).reshape(
                self.input_video.shape[1], self.input_video.shape[2], 4)

    def create_mask_pixmap(self, input_mask_slide):
        self.mask_qimage_view[..., 3] = input_mask_slide*int(1*255)
        self.current_mask_pixmap = QtGui.QPixmap.fromImage(self.mask_qimage)
        return self.current_mask_pixmap

    def create_pixmap(self, input_file):
        input_file = Normalization(video=input_file).normalized_image_specific()
        self.input_file = input_file
        if self.parent().parent().alpha is None:
            input_file = self.con_adj.auto_pixelTransforms(input_file)
        else:

            alpha = self.parent().parent().alpha
            beta = self.parent().parent().beta
            min_intensity = self.parent().parent().min_intensity
            max_intensity = self.parent().parent().max_intensity
            input_file = self.con_adj.pixel_transforms(input_file, alpha, beta, min_intensity, max_intensity)

        if self.medianFilterFlag:
            input_file = median_filter(input_file, 3)

        if self.parent().parent().title != "PNG":
            if input_file.shape[0] >= input_file.shape[1]:
                widthStep = input_file.shape[1]
                img = QtGui.QImage(input_file, input_file.shape[1], input_file.shape[0], widthStep,
                                   QtGui.QImage.Format_Indexed8)
                img.scaledToWidth(input_file.shape[0])
            else:
                widthStep = input_file.shape[1]
                img = QtGui.QImage(input_file, input_file.shape[1], input_file.shape[0], widthStep,
                                   QtGui.QImage.Format_Indexed8)

            newimg = img.convertToFormat(QtGui.QImage.Format.Format_RGB888)
            self.current_pixmap = QtGui.QPixmap(newimg)
            self.update_slice(self.current_pixmap)
        elif self.parent().parent().title == "PNG":
            self.current_pixmap = QtGui.QPixmap(input_file)

        return self.current_pixmap

    def update_slice(self, input_pixmap):
        self.t = input_pixmap
        self._zoom = 0
        width = np.max([self.video_width, 400])
        height = np.max([self.video_height, 400])

        self.pixmap_video = input_pixmap.scaled(QtCore.QSize(width, height), QtCore.Qt.KeepAspectRatio)
        self._photo = self.scene.addPixmap(self.pixmap_video)
        if self.livePaint_Flag:
            self.paint()

    def update_overlay(self, input_mask_pixmap, first_flag=True):
        width = np.max([self.video_width, 400])
        height = np.max([self.video_height, 400])
        self.scaled_mask_pixmap = input_mask_pixmap.scaled(QtCore.QSize(width, height), QtCore.Qt.KeepAspectRatio)

        self._photo.setPixmap(self.scaled_mask_pixmap)
        self._photo_mask = self.scene.addPixmap(self.scaled_mask_pixmap)

        if self.parent().parent().title != "PNG" and first_flag is True:

            self.parent().parent().slice_slider.setValue(self.slice_num)
            self.current_pixmap = self.create_pixmap(self.input_video[self.slice_num, :, :])
            self.update_slice(self.current_pixmap)
            if self.mask_is_set is True:
                self.current_mask_pixmap = self.create_mask_pixmap(self.maskArray[self.slice_num, :, :])
                self.update_overlay(self.current_mask_pixmap, first_flag=False)

    def wheelEvent(self, event):
        modifiers = QtWidgets.QApplication.keyboardModifiers()
        if modifiers == QtCore.Qt.ControlModifier:
            if event.angleDelta().y() > 0:
                factor = 1.25
                self._zoom += 1
            elif event.angleDelta().y() < 0:
                factor = 0.75
                self._zoom -= 1
            else:
                factor = 0.8
                self._zoom -= 1

            if self._zoom > 0:
                self.scale(factor, factor)
            elif self._zoom < 0:
                self.scale(factor, factor)
            else:
                self._zoom = 0

        else:
            if self.parent().parent().title == "PNG":
                pass
            elif self.parent().parent().title != "PNG":
                steps = event.delta() // 120
                vector = steps and steps // abs(steps)  # 0, 1, or -1
                for step in range(1, abs(steps) + 1):

                    self.slice_num -= vector
                    self.parent().parent().slice_slider.setValue(self.slice_num)
                    self.current_pixmap = self.create_pixmap(self.input_video[self.slice_num, :, :])
                    self.update_slice(self.current_pixmap)
                    if self.mask_is_set is True:
                        self.current_mask_pixmap = self.create_mask_pixmap(self.maskArray[self.slice_num, :, :])
                        self.update_overlay(self.current_mask_pixmap)

    @QtCore.Slot()
    def active_LivePaint(self):

        if self.livePaint_Flag is False:
            self.livePaint_Flag = True
        elif self.livePaint_Flag is True:
            self.livePaint_Flag = False
            try:
                self.scene.removeItem(self.Line)
                self.scene.update()
            except:
                pass

    def paint(self):
        pen = QPen(Qt.red)
        pen.setCosmetic(True)
        pen.setWidth(1)

        try:
            self.scene.removeItem(self.Line)
        except AttributeError:
            pass

        self.Line = self.scene.addLine(self.x0, self.y0, self.x1, self.y1, pen)
        self.scene.addItem(self.Line)
        self.scene.update()
        self.read_pixel_valus()

    def read_pixel_valus(self):
        try:
            DI = DirectionalIntensity()
            radial_index = DI.interpolate_pixels_along_line(x0=int(self.ori_X0), y0=int(self.ori_Y0), x1=int(self.ori_X1),
                                                            y1=int(self.ori_Y1))
            frame = self.RAW_Video[self.slice_num, ...]

            pixels_ = []
            for idx_ in radial_index:
                pixels_.append(frame[int(idx_[1]), int(idx_[0])])

            self.updatePlot.update_plot(pixels_)
            self.updatePlot.show()
        except:
            pass

















import numpy as np
from scipy.ndimage import gaussian_filter, gaussian_laplace
import math
from math import sqrt, log
from scipy import spatial
from skimage.util import img_as_float


def dog_preview(images, min_sigma, max_sigma, sigma_ratio):

    min_range = []
    max_range = []

    for idx_ in range(images.shape[0]):
        image = images[idx_, ...]
        image = img_as_float(image)

        # if both min and max sigma are scalar, function returns only one sigma
        scalar_sigma = np.isscalar(max_sigma) and np.isscalar(min_sigma)

        # Gaussian filter requires that sequence-bin_type sigmas have same
        # dimensionality as image. This broadcasts scalar kernels
        if np.isscalar(max_sigma):
            max_sigma = np.full(image.ndim, max_sigma, dtype=float)
        if np.isscalar(min_sigma):
            min_sigma = np.full(image.ndim, min_sigma, dtype=float)

        # Convert sequence types to array
        min_sigma = np.asarray(min_sigma, dtype=float)
        max_sigma = np.asarray(max_sigma, dtype=float)

        # k such that min_sigma*(sigma_ratio**k) > max_sigma
        k = int(np.mean(np.log(max_sigma / min_sigma) / np.log(sigma_ratio) + 1))

        # a geometric progression of standard deviations for gaussian kernels
        sigma_list = np.array([min_sigma * (sigma_ratio ** i)
                               for i in range(k + 1)])

        gaussian_images = [gaussian_filter(image, s) for s in sigma_list]

        # computing difference between two successive Gaussian blurred images
        # multiplying with average standard deviation provides scale invariance
        dog_images = [(gaussian_images[i] - gaussian_images[i + 1])
                      * np.mean(sigma_list[i]) for i in range(k)]

        image_cube = np.stack(dog_images, axis=-1)
        min_range.append(image_cube.min())
        max_range.append(image_cube.max())

    min_range_ = np.min(min_range)
    max_range_ = np.max(max_range)

    return min_range_, max_range_